"""
main shutter and branch shutter functions
"""
from time import sleep
from epics import caget, caput

import iexcode.instruments.cfg as iex
from iexcode.instruments.utilities import dateandtime


def shutter_check(verbose=True):
    """
    Checks main shutter is open, if not opens it"
    Checks brancg shutter is open, if not opens it"
    
    Previously: Check_MainShutter
    """
    while True:
        shutter_open = main_shutter_status()
        if shutter_open == False:
            print("MAIN SHUTTER CLOSED !!!" , dateandtime())
            main_shutter_open()
            sleep(10)
        else:
            if verbose:
                print("Shutter is now open" , dateandtime())
            break
        shutter_open = branch_shutter_status()
        
        if shutter_open == False:
            branch_shutter_open()

##############################################################################################################
################################             main shutter             ##############################
##############################################################################################################

def main_shutter_open(verbose=True):
    """
    opens the main shutter

    Previously: Open_MainShutter 
    """
    caput("PC:29ID:FES_OPEN_REQUEST.VAL",1, wait=True,timeout=180000)
    if verbose:
        print("Opening Main Shutter...")

def main_shutter_close(verbose=True):
    """
    closes the main shutter

    Previously: Close_MainShutter
    """
    caput("PC:29ID:FES_CLOSE_REQUEST.VAL",1,wait=True,timeout=18000)
    if verbose:
        print("Closing Main Shutter...")

def main_shutter_status():
    """
    checks on the status of the main shutter
    and returns shutter_open = True / False

    """
    SS1=caget("EPS:29:ID:SS1:POSITION")
    SS2=caget("EPS:29:ID:SS2:POSITION")
    PS2=caget("EPS:29:ID:PS2:POSITION")
    check=SS1*SS2*PS2

    if check == 8:
        shutter_open = True
    else:
        shutter_open = False
    
    return shutter_open


##############################################################################################################
################################             branch shutters            ##############################
##############################################################################################################

def branch_shutter_status(branch=None,verbose=False):
    """
    checks on the status of the main shutter
    and returns shutter_open = True / False
    """
    if branch == None:
        branch = iex.BL.branch
    pvA="PA:29ID:S"+branch.upper()+"S_BLOCKING_BEAM.VAL"
    pvB="PB:29ID:S"+branch.upper()+"S_BLOCKING_BEAM.VAL"
    #"ON" = 1 => shutter open
    status=caget(pvA)+caget(pvB)
    if status == 2:
        shutter_open=True
    else:
        shutter_open=False

    if verbose:
        status = 'Open' if shutter_open else 'Closed'
        print(branch+"-shutter is "+status)
    return shutter_open

def branch_shutter_close(branch=None,verbose=False):
    """
    closes current branch shutter

    Previously: Close_BranchShutter
    """
    if branch == None:
        branch = iex.BL.branch
    caput("PC:29ID:S"+branch.upper()+"S_CLOSE_REQUEST.VAL",1,wait=True,timeout=18000)
    print("Closing "+branch.upper()+"-Shutter...")


def branch_shutter_open(branch=None,verbose=False):
    """
    Opens current branch shutter 

    Previoulsy: Open_BranchShutter

    """
    if branch == None:
        branch = iex.BL.branch
    shutter_status = branch_shutter_status()
    if shutter_status:
        print(branch+"-Shutter already open...")
    else:
        caput("PC:29ID:S"+branch.upper()+"S_OPEN_REQUEST.VAL",1,wait=True,timeout=18000)
        print("Opening "+branch.upper()+"-Shutter...")
        
